###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__author__ = '$Author: morten $'
__version__ = '$Revision: 1.12 $'


import gateway
import smtplib
import fetchmail_box
import rfc822_message_parse

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import base_item


manage_add_mail_gateway_form = HTMLFile('mail_gateway_add', globals())
add_mail_gateway_redirect = HTMLFile('mail_gateway_add_redirect', globals())


def manage_add_mail_gateway(self, id, server, protocol, port,
							smtp_server, smtp_port, REQUEST=None):
	"Add an instance of the mail_gateway class."""

	self._setObject(id, mail_gateway(id, server, protocol, port,
									 smtp_server, smtp_port))

	if REQUEST is not None:

		return self.manage_main(REQUEST)
				

class mail_gateway(gateway.gateway,
				   Products.ZCatalog.CatalogAwareness.CatalogAware,
				   OFS.Folder.Folder,
				   Persistent,
				   Acquisition.Implicit,
				   AccessControl.Role.RoleManager,
				   base_item.base_item):

	meta_type = 'MailGateway'
	icon = 'misc_/GUM/Folder_icon.gif'

	_properties = (
		{'id': 'server', 'type':'string'},
		{'id': 'protocol', 'type':'string'},
		{'id': 'port', 'type':'string'},
		{'id': 'smtp_server', 'type':'string'},
		{'id': 'smtp_port', 'type':'string'},
		{'id': 'options', 'type':'multiline'}
		)

	meta_types = None

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View MailGateway',
		 ('',)),
		)

	index_html = HTMLFile('mail_gateway_index')

	protocols = ('imap4','pop2','pop3','auto')

	option_descriptions = {'bounce':'Bounce \'illegal\' mails.',
						   'ssl':'Enable SSL login/transfer.',
						   'keep':'Don\' delete downloaded messages.',
						   'size_limit':'Maximum message size.',
						   'retrieval_limit':'Maximum messages retrieved.'
						   }

	spam_error_codes = {}


	def __init__(self,server,protocol,port=None,
				 smtp_server=None,smtp_port=None,
				 username='',password='',options=[]):
		"""The type argument can be either POP2, POP3, IMAP or AUTO"""

		gateway.gateway.__init__(self,server,port)

		self.options = options

		if protocol not in mail_gateway.protocols:

			raise 'ValueError','Protocol "%s" not supported' % protocol
        
		self.protocol = protocol
		self.port = port

		# If the SMTP server isn't specified, we assume that it is
		# located at the same host as the mail account.

		if smtp_server:

			self.smtp_server = smtp_server

		else:
            
			self.smtp_server = self.server

		self.smtp_port = smtp_port

		self.username = username
		self.password = password

		self.smtp_connection = smtplib.SMTP(self.smtp_server,self.smtp_port)

		self.index_object()


	def build_smtp(self):
		"""Reopens any of the connections if necessary."""

		if self.port:

			return self.SMTP(self.server,self.port)

		else:

			return self.SMTP(self.server)


	def refresh_smtp(self):
		"""If we've lost the connection, open it up again."""

		if not self.smtp_connection.sock:

			self.build_smtp()
        

	def retrieve(self,fetchall=[]):
		"""Retrieves a mail message and instantiates an rfc822_message
		object."""

		if fetchall:

			fetchall = ['fetchall']

		msgs = fetchmail_box.build_box(
			self.server,
			self.username,self.password,
			self.options + fetchall,
			protcol = self.protocol)

		"""
		msgs = []
		index = 0

		while 1:

			msg = mbox.next()

			if msg:

				msgs.append(rfc822_message_parse.rfc822_message_parse(msg))

			else:
                
				print msg,'in retrieve()'
				break

			print index

		"""

		return msgs


	def post(self,message):
		"""Deconstructs an rfc822_message object and posts it."""

		con = self.build_smtp()

		self.smtp_connection.sendmail(message.sender(),message.recipients(),
									  message.raw_message())

		con.quit()
